use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007002
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
PadWalker
Tcl's
Welch
uplevel
Michael
Schwern
mschwern
David
Golden
dagolden
and
Adam
Kennedy
adamk
Alexandr
Ciornii
alexchorny
xdg
Nick
Koston
nick
Gray
mg13
lib
Sub
Uplevel
